@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2017-2019 by the Free Software Foundation, Inc.
+# Copyright (C) 2017-2021 by the Free Software Foundation, Inc.
 #
 # This file is part of Postorius.
 #
@@ -20,25 +20,21 @@
 import re
 
 from django import forms
+from django.apps import apps
 from django.core.exceptions import ValidationError
 from django.core.validators import validate_email
 from django.utils.translation import gettext_lazy as _
 
 from django_mailman3.lib.mailman import get_mailman_client
 
-from postorius.forms.fields import ListOfStringsField
+from postorius.forms.fields import (
+    ACTION_CHOICES, ListOfStringsField, delivery_mode_field,
+    delivery_status_field, moderation_action_field)
+from postorius.forms.validators import validate_uuid_or_email
 from postorius.models import EmailTemplate, _email_template_help_text
 from postorius.utils import LANGUAGES
 
 
-ACTION_CHOICES = (
-    ("hold", _("Hold for moderation")),
-    ("reject", _("Reject (with notification)")),
-    ("discard", _("Discard (no notification)")),
-    ("accept", _("Accept immediately (bypass other rules)")),
-    ("defer", _("Default processing")),
-)
-
 DIGEST_FREQUENCY_CHOICES = (
     ("daily", _("Daily")),
     ("weekly", _("Weekly")),
@@ -144,10 +140,16 @@ class ListSubscribe(forms.Form):
     """Form fields to join an existing list.
     """
 
-    email = forms.ChoiceField(
+    DELIVERY_STATUS_CHOICES = (("enabled", _('Enabled')),
+                               ("by_user", _('Disabled')))
+
+    subscriber = forms.ChoiceField(
         label=_('Your email address'),
-        validators=[validate_email],
         widget=forms.Select(),
+        validators=[validate_uuid_or_email, ],
+        help_text=_(
+            'Subscribing via "Primary Address" will change subscription'
+            ' address when you change your primary address.'),
         error_messages={
             'required': _('Please enter an email address.'),
             'invalid': _('Please enter a valid email address.')})
@@ -155,10 +157,19 @@ class ListSubscribe(forms.Form):
     display_name = forms.CharField(
         label=_('Your name (optional)'), required=False)
 
-    def __init__(self, user_emails, *args, **kwargs):
+    delivery_mode = delivery_mode_field(default='regular')
+    delivery_status = delivery_status_field(choices=DELIVERY_STATUS_CHOICES,
+                                            widget=forms.RadioSelect)
+
+    def __init__(self, user_emails, user_id, primary_email, *args, **kwargs):
         super(ListSubscribe, self).__init__(*args, **kwargs)
-        self.fields['email'].choices = ((address, address)
-                                        for address in user_emails)
+        choices = list((address, address)
+                       for address in user_emails)
+        if primary_email and user_id:
+            choices.insert(
+                0,
+                (user_id, _('Primary Address ({})').format(primary_email)))
+        self.fields['subscriber'].choices = choices
 
 
 class ListAnonymousSubscribe(forms.Form):
@@ -225,6 +236,12 @@ class MemberPolicyForm(ListSettingsForm):
 class BounceProcessingForm(ListSettingsForm):
     """List's bounce processing settings."""
 
+    forward_unrecognized_choices = (
+        ('discard', _('Discard')),
+        ('administrators', _('List Admins')),
+        ('site_owner', _('Site Admin')),
+    )
+
     process_bounces = forms.BooleanField(
         widget=forms.RadioSelect(choices=((True, _('Yes')), (False, _('No')))),
         required=False,
@@ -251,6 +268,15 @@ class BounceProcessingForm(ListSettingsForm):
             ' the interim, the bounce score is reset to zero.'
             ' This value must be an integer. '))
 
+    bounce_notify_owner_on_bounce_increment = forms.BooleanField(
+        widget=forms.RadioSelect(choices=((True, _('Yes')), (False, _('No')))),
+        required=False,
+        label=_('Notify owner on bounce increment'),
+        help_text=_(
+            'This option controls whether or not the list owner is notified'
+            ' when a member\'s bounce score is incremented, but to a value'
+            ' less than their bounce threshold. '))
+
     bounce_notify_owner_on_disable = forms.BooleanField(
         widget=forms.RadioSelect(choices=((True, _('Yes')), (False, _('No')))),
         required=False,
@@ -269,7 +295,13 @@ class BounceProcessingForm(ListSettingsForm):
             'notified when a member is removed from the list after '
             'their disabled notifications have been exhausted. '))
 
-    # forward_unrecognized_bounces_to =
+    forward_unrecognized_bounces_to = forms.ChoiceField(
+        choices=forward_unrecognized_choices,
+        widget=forms.RadioSelect,
+        label=_('Forward unrecognized bounces'),
+        help_text=_('Discard: Unrecognized bounces will be discarded\n'
+                    'List Admins: Send to the list owners and moderators\n'
+                    'Site Admin: Send to the site\'s configured site_owner'))
 
     bounce_you_are_disabled_warnings_interval = forms.CharField(
         label=_('Bounce disabled warnings interval'),
@@ -301,6 +333,11 @@ class ArchiveSettingsForm(ListSettingsForm):
         ("never", _("Do not archive this list")),
     )
 
+    archive_rendering_mode_choices = (
+        ("text", _("Plain text")),
+        ("markdown", _("Markdown text")),
+    )
+
     archive_policy = forms.ChoiceField(
         choices=archive_policy_choices,
         widget=forms.RadioSelect,
@@ -313,6 +350,17 @@ class ArchiveSettingsForm(ListSettingsForm):
         label=_('Active archivers'),
         required=False)  # May be empty if no archivers are desired.
 
+    archive_rendering_mode = forms.ChoiceField(
+        choices=archive_rendering_mode_choices,
+        widget=forms.RadioSelect,
+        required=False,
+        label=_('Archive Rendering mode'),
+        help_text=_('This option enables rendering of emails in archiver as '
+                    'rich text with formatting based on markup in the email.'
+                    '\nCurrently, this option is only supported by Hyperkitty.'
+                    )
+    )
+
     def __init__(self, *args, **kwargs):
         super(ArchiveSettingsForm, self).__init__(*args, **kwargs)
         archiver_opts = sorted(self._mlist.archivers.keys())
@@ -321,6 +369,10 @@ class ArchiveSettingsForm(ListSettingsForm):
         if self.initial:
             self.initial['archivers'] = [
                 key for key in archiver_opts if self._mlist.archivers[key] is True]   # noqa
+        # If Hyperkitty isn't installed, do not show the archive_rendering_mode
+        # field since it doesn't apply to other archivers.
+        if not apps.is_installed('hyperkitty'):
+            del self.fields['archive_rendering_mode']
 
     def clean_archivers(self):
         result = {}
@@ -395,6 +447,16 @@ class MessageAcceptanceForm(ListSettingsForm):
             'is matched against the list of explicitly accepted, held, '
             'rejected (bounced), and discarded addresses. '
             'If no match is found, then this action is taken.'))
+    emergency = forms.BooleanField(
+        widget=forms.RadioSelect(choices=((True, _('Yes')), (False, _('No')))),
+        required=False,
+        label=_('Emergency Moderation'),
+        help_text=_(
+            'When this option is enabled, all list traffic is emergency'
+            ' moderated, i.e. held for moderation. Turn this option on when'
+            ' your list is experiencing a flamewar and you want a cooling off'
+            ' period. '),
+    )
     max_message_size = forms.IntegerField(
         min_value=0,
         label=_('Maximum message size'),
@@ -408,7 +470,8 @@ class MessageAcceptanceForm(ListSettingsForm):
         label=_('Maximum number of recipients'),
         required=False,
         help_text=_(
-            'The maximum number of recipients for a message. '
+            'If a post has this many or more explicit recipients (To: and '
+            'Cc:), the post will be held for moderation. '
             'This can be used to prevent mass mailings from being accepted. '
             'A value of 0 disables the check.'))
 
@@ -627,7 +690,7 @@ class AlterMessagesForm(ListSettingsForm):
         label=_('Filter types'),
         required=False,
         help_text=_(
-            'MimeTypes to filter from the incoming posts. A list of common '
+            'MIME types to filter from the incoming posts. A list of common '
             'types can be found '
             '<a href="https://developer.mozilla.org/en-US/docs/Web/HTTP/Basics_of_HTTP/MIME_types/Common_types">here </a>'  # noqa# E501
             ))
@@ -641,7 +704,7 @@ class AlterMessagesForm(ListSettingsForm):
         label=_('Pass types'),
         required=False,
         help_text=_(
-            'MimeTypes to allow in the incoming posts. A list of common '
+            'MIME types to allow in the incoming posts. A list of common '
             'types can be found '
             '<a href="https://developer.mozilla.org/en-US/docs/Web/HTTP/Basics_of_HTTP/MIME_types/Common_types">here </a>'  # noqa# E501
             ))
@@ -846,6 +909,16 @@ class ListAutomaticResponsesForm(ListSettingsForm):
             'is most useful for transparently migrating lists from some other '
             'mailing list manager to Mailman.\n'
             'The text of Welcome message can be set via the Templates tab.'))
+    send_goodbye_message = forms.ChoiceField(
+        choices=((True, _('Yes')), (False, _('No'))),
+        widget=forms.RadioSelect,
+        required=False,
+        label=_('Send goodbye message'),
+        help_text=_(
+            'Send goodbye message to newly unsubscribed members? '
+            'Turn this off only if you plan on unsubscribing people manually '
+            'and don\'t want them to know that you did so.\n'
+            'The text of Goodbye message can be set via the Templates tab.'))
     admin_immed_notify = forms.BooleanField(
         widget=forms.RadioSelect(choices=((True, _('Yes')), (False, _('No')))),
         required=False,
@@ -934,7 +1007,7 @@ class ListIdentityForm(ListSettingsForm):
         help_text=_('Flag indicating that posts to the list should be gated to'
                     ' the linked newsgroup.')
     )
-    linked_newgroup = forms.CharField(
+    linked_newsgroup = forms.CharField(
         label=_('Linked Newsgroup'),
         required=False,
         help_text=_(
@@ -1008,6 +1081,40 @@ class ListMassSubscription(forms.Form):
         widget=forms.CheckboxInput()
         )
 
+    invitation = forms.BooleanField(
+        label=_('Invitation'),
+        initial=False,
+        required=False,
+        help_text=_(
+            'If checked, the other checkboxes are ignored and the users will '
+            'be sent an invitation to join the list and will be subscribed '
+            'upon acceptance thereof.'),
+        widget=forms.CheckboxInput()
+        )
+
+    send_welcome_message = forms.ChoiceField(
+        choices=((True, _('Yes')),
+                 (False, _('No')),
+                 ('default', _('List default'))),
+        widget=forms.RadioSelect,
+        initial='default',
+        required=False,
+        label=_('Send welcome message'),
+        help_text=_(
+            'If set to "Yes" or "No", List\'s default setting of '
+            'send_welcome_message will be ignored for these subscribers and a'
+            ' welcome message will be sent or not sent based on the choice.'),
+        )
+
+    def clean_send_welcome_message(self):
+        """Choose from True or False. Any other value is equivalent to None."""
+        data = self.cleaned_data['send_welcome_message']
+        if data in ('True', 'False'):
+            return data
+        # None implies this value is unset and isn't passed on to Core in API
+        # call.
+        return None
+
 
 class ListMassRemoval(forms.Form):
 
@@ -1077,36 +1184,32 @@ class MemberModeration(forms.Form):
     """
     Form handling the member's moderation_action.
     """
-    moderation_action = forms.ChoiceField(
-        widget=forms.Select(),
-        label=_('Moderation'),
-        required=False,
-        choices=[(None, _('List default'))] + list(ACTION_CHOICES),
-        help_text=_(
-            'Default action to take when this member posts to the list. \n'
-            'List default -- follow the list\'s default member action. \n'
-            'Hold -- This holds the message for approval by the list '
-            'moderators. \n'
-            'Reject -- this automatically rejects the message by sending a '
-            'bounce notice to the post\'s author. The text of the bounce '
-            'notice can be configured by you. \n'
-            'Discard -- this simply discards the message, with no notice '
-            'sent to the post\'s author. \n'
-            'Accept -- accepts any postings without any further checks. \n'
-            'Default Processing -- run additional checks and accept '
-            'the message. \n'))
+    moderation_action = moderation_action_field()
 
 
 class ChangeSubscriptionForm(forms.Form):
-    email = forms.ChoiceField()
 
-    def __init__(self, user_emails, *args, **kwargs):
+    subscriber = forms.ChoiceField(
+        label=_('Select Email'),
+        required=False,
+        widget=forms.Select(),
+        validators=[validate_uuid_or_email, ],)
+
+    member_id = forms.CharField(
+        required=True,
+        label="",
+        widget=forms.TextInput(attrs={'readonly': True, 'hidden': True}))
+
+    def __init__(self, user_emails, user_id, primary_email,
+                 *args, **kwargs):
         super(ChangeSubscriptionForm, self).__init__(*args, **kwargs)
-        self.fields['email'] = forms.ChoiceField(
-            label=_('Select Email'),
-            required=False,
-            widget=forms.Select(),
-            choices=((address, address) for address in user_emails))
+        choices = list((address, address)
+                       for address in user_emails)
+        if primary_email and user_id:
+            choices.insert(
+                0,
+                (user_id, _('Primary Address ({})').format(primary_email)))
+        self.fields['subscriber'].choices = choices
 
 
 class TemplateUpdateForm(forms.ModelForm):
@@ -1120,3 +1223,11 @@ class TemplateUpdateForm(forms.ModelForm):
     class Meta:
         model = EmailTemplate
         fields = ['data']
+
+
+class TokenConfirmForm(forms.Form):
+    """Form to confirm pending (un)subscription requests from User."""
+    token = forms.CharField(
+        required=True,
+        label="",
+        widget=forms.TextInput(attrs={'readonly': True, 'hidden': True}))
